// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;

namespace Smdn.Formats.Ini {
  public class Document : IEnumerable<Section> {
    public Section this[string name] {
      get
      {
        if (!sections.ContainsKey(name)) {
          AppendSection(name);
        }

        return sections[name];
      }
    }

    public Section[] Sections
    {
      get { return (new List<Section>(sections.Values)).ToArray(); }
    }

    public Document()
    {
      sections = new Dictionary<string, Section>();

      // append default section
      AppendSection(string.Empty);
    }

    public static Document Load(string path)
    {
      using (var reader = new StreamReader(path)) {
        return Load(reader);
      }
    }

    public static Document Load(TextReader reader)
    {
      return Parser.Parse(reader);
    }

    public void Save(string path)
    {
      using (var writer = new StreamWriter(path)) {
        Save(writer);
      }
    }

    public void Save(TextWriter writer)
    {
      Formatter.Format(this, writer);
    }

    IEnumerator IEnumerable.GetEnumerator()
    {
      return GetEnumerator();
    }

    public IEnumerator<Section> GetEnumerator()
    {
      return sections.Values.GetEnumerator();
    }

    private void AppendSection(string name)
    {
      var section = new Section(name);

      sections.Add(section.Name, section);
    }

    public override string ToString()
    {
      return string.Format("{{Sections={0}}}", sections.Count);
    }

    private Dictionary<string, Section> sections;
  }
}
