// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using Microsoft.Win32;

using System;
using System.IO;

namespace Smdn.Formats.Mime {
  public static class MimeType {
    public static string GetMimeTypeByExtension(string mimeTypesFile, string extension)
    {
      if (EnvironmentExtension.IsRunningOnUnix)
        return GetMimeTypeByExtensionUnix(mimeTypesFile, extension);
      else
        return GetMimeTypeByExtensionWin(extension);
    }

    private static string GetMimeTypeByExtensionUnix(string mimeTypesFile, string extension)
    {
      if (extension.StartsWith("."))
        extension = extension.Substring(1);

      using (var reader = new StreamReader(mimeTypesFile)) {
        for (;;) {
          var line = reader.ReadLine();

          if (line == null)
            break;
          else if (line.StartsWith("#"))
            continue;

          var entry = line.Split(new[] {"\t", " "}, StringSplitOptions.RemoveEmptyEntries);

          if (entry.Length <= 1)
            continue;

          for (var index = 1; index < entry.Length; index++) {
            if (entry[index] == extension)
              return entry[0];
          }
        }
      }

      return null;
    }
    
    private static string GetMimeTypeByExtensionWin(string extension)
    {
      var key = Registry.ClassesRoot.OpenSubKey(extension);

      if (key == null)
        return null;

      var mimeType = key.GetValue("Content Type");

      if (mimeType == null)
        return null;
      else
        return (string)mimeType;
    }
  }
}