// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Text;
using System.Net;
using System.Security.Cryptography;

namespace Smdn.Protocols.Imap4.Client.Protocol.Authentication {
  /*
   * RFC 2195 IMAP/POP AUTHorize Extension for Simple Challenge/Response
   * http://tools.ietf.org/html/rfc2195
   */
  internal class CRAMMD5AuthenticationModule : BinaryAuthenticationModuleBase {
    public override ImapAuthenticationMechanism Mechanism {
      get { return ImapAuthenticationMechanism.CRAMMD5; }
    }

    protected override byte[] ExchangeChallengeResponse(byte[] challenge, ImapConnection connection, NetworkCredential credential)
    {
      // 2. Challenge-Response Authentication Mechanism (CRAM)

      // The data encoded in the first ready response contains an
      // presumptively arbitrary string of random digits, a timestamp, and the
      // fully-qualified primary host name of the server.  The syntax of the
      // unencoded form must correspond to that of an RFC 822 'msg-id'
      // [RFC822] as described in [POP3].

      // The client makes note of the data and then responds with a string
      // consisting of the user name, a space, and a 'digest'.  The latter is
      // computed by applying the keyed MD5 algorithm from [KEYED-MD5] where
      // the key is a shared secret and the digested text is the timestamp
      // (including angle-brackets).
      //
      // This shared secret is a string known only to the client and server.
      var keyed = (new HMACMD5(Encoding.ASCII.GetBytes(credential.Password ?? string.Empty))).ComputeHash(challenge);

      // The `digest' parameter itself is a 16-octet value which is sent in
      // hexadecimal format, using lower-case ASCII characters.
      var digest = ToLowerCaseHexString(keyed);

      return Encoding.ASCII.GetBytes(string.Format("{0} {1}", credential.UserName, digest));
    }
  }
}