// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

namespace Smdn.Protocols.Imap4.Client.Protocol {
  // handles '4. Data Formats'
  public class ImapData {
    public ImapDataFormat Format {
      get { return format; }
    }

    // handles '4.1. Atom'
    //         '4.2. Number'
    //         '4.3. String'
    public string Text {
      get { return text; }
    }

    // handles '4.4. Parenthesized List'
    public ImapData[] List {
      get { return list; }
    }

    private ImapData(string text)
    {
      // 4.1. Atom, 4.2. Number, 4.3. String
      this.text = text;
      this.list = null;
      this.format = ImapDataFormat.Text;
    }

    private ImapData(ImapData[] list)
    {
      // 4.4. Parenthesized List
      this.text = null;
      this.list = list;
      this.format = ImapDataFormat.List;
    }

    private ImapData()
    {
      // 4.5. NIL
      this.text = null;
      this.list = null;
      this.format = ImapDataFormat.Nil;
    }

    internal static ImapData CreateTextData(string text)
    {
      if (text == null)
        throw new ArgumentNullException("text");

      return new ImapData(text);
    }

    internal static ImapData CreateListData(ImapData[] list)
    {
      if (list == null)
        throw new ArgumentNullException("list");

      return new ImapData(list);
    }

    internal static ImapData CreateNilData()
    {
      return new ImapData();
    }

    public override string ToString()
    {
      switch (format) {
        case ImapDataFormat.Text:
          return string.Format("{{Text:{0}}}", text);
        case ImapDataFormat.List:
          return string.Format("{{List:{0}}}", string.Join(", ", Array.ConvertAll(list, delegate(ImapData d) {
            return d.ToString();
          })));
        case ImapDataFormat.Nil:
          return "{NIL}";
        default:
          return "{unknown}";
      }
    }

    private ImapDataFormat format = ImapDataFormat.None;
    private string text = string.Empty;
    private ImapData[] list = null;
  }
}