// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#if DUMP
#define TRACING
#endif

#if TRACE
#define TRACING
#endif

using System;
using System.IO;
using System.Text;

namespace Smdn.Protocols.Imap4.Client.Protocol {
#if TRACING
  internal class ImapTracingStream : System.IO.Stream {
    public Stream BaseStream {
      get { return baseStream; }
    }

    public override bool CanSeek {
      get { return baseStream.CanSeek; }
    }

    public override bool CanRead {
      get { return baseStream.CanWrite; }
    }

    public override bool CanWrite {
      get { return baseStream.CanRead; }
    }

    public override bool CanTimeout {
      get { return baseStream.CanTimeout; }
    }

    public override long Position {
      get { return baseStream.Position; }
      set { baseStream.Position = value; }
    }

    public override long Length {
      get { return baseStream.Length; }
    }

    public ImapTracingStream(ImapConnection connection, Stream baseStream)
    {
      this.connection = connection;
      this.baseStream = baseStream;
    }

    public override void Close()
    {
      Dispose(true);
    }

    protected override void Dispose(bool disposing)
    {
      if (disposed)
        return;

      baseStream = null;
      connection = null;

      disposed = true;
    }

    public override void SetLength(long @value)
    {
      baseStream.SetLength(@value);
    }

    public override long Seek(long offset, SeekOrigin origin)
    {
      return baseStream.Seek(offset, origin);
    }

    public override void Flush()
    {
      baseStream.Flush();
    }

    public override int Read(byte[] dest, int offset, int count)
    {
      var ret = baseStream.Read(dest, offset, count);

      Trace.LogReceived(connection, dest, offset, ret);

      return ret;
    }

    public override void Write(byte[] src, int offset, int count)
    {
      Trace.LogSent(connection, src, offset, count);

      baseStream.Write(src, offset, count);
    }

    private ImapConnection connection;
    private Stream baseStream;
    private bool disposed;
  }
#endif
}