// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

namespace Smdn.Protocols.Imap4.Client.Session {
  internal class ImapMailboxManager {
    public ImapMailboxManager(ImapSession session)
    {
      this.session = session;
    }

    public ImapMailbox GetExistOrCreate(ImapMailboxList mailboxList)
    {
      var mailbox = GetExistOrCreate(mailboxList.Name);

      mailbox.Flags = mailboxList.NameAttributes;
      mailbox.HierarchyDelimiter = mailboxList.HierarchyDelimiter;

      return mailbox;
    }

    public ImapMailbox GetExistOrCreate(string name)
    {
      if (name.Equals("INBOX", StringComparison.InvariantCultureIgnoreCase))
        name = "INBOX";

      if (mailboxes.ContainsKey(name))
        return mailboxes[name];
      else
        return Add(new ImapMailbox(name));
    }

    public ImapMailbox GetExist(string mailboxName)
    {
      if (mailboxes.ContainsKey(mailboxName))
        return mailboxes[mailboxName];
      else
        return null;
    }

    public ImapMailbox Add(ImapMailbox mailbox)
    {
      mailbox.Session = session;

      mailboxes.Add(mailbox.Name, mailbox);

      return mailbox;
    }

    public void Delete(string mailboxName)
    {
      if (mailboxName.Equals("INBOX", StringComparison.InvariantCultureIgnoreCase))
        return;

      if (mailboxes.ContainsKey(mailboxName)) {
        var mailbox = mailboxes[mailboxName];

        mailbox.Session = null;

        mailboxes.Remove(mailboxName);
      }
    }

    public ImapMailbox Rename(string existingMailboxName, string newMailboxName)
    {
      var mailbox = GetExist(existingMailboxName);

      if (mailbox == null)
        throw new InvalidOperationException("mailbox not exist");

      // 6.3.5. RENAME Command
      //       Renaming INBOX is permitted, and has special behavior.  It moves
      //       all messages in INBOX to a new mailbox with the given name,
      //       leaving INBOX empty.  If the server implementation supports
      //       inferior hierarchical names of INBOX, these are unaffected by a
      //       rename of INBOX.
      if (mailbox.Name.Equals("INBOX", StringComparison.InvariantCultureIgnoreCase)) {
        mailboxes.Remove(mailbox.Name);
        mailbox.Name = newMailboxName;
        mailboxes.Add(mailbox.Name, mailbox);

        Add(new ImapMailbox("INBOX"));

        return mailbox;
      }
      else {
        mailboxes.Remove(mailbox.Name);
        mailbox.Name = newMailboxName;
        mailboxes.Add(mailbox.Name, mailbox);

        return mailbox;
      }
    }

    private /*readonly*/ ImapSession session;
    private /*readonly*/ Dictionary<string, ImapMailbox> mailboxes = new Dictionary<string, ImapMailbox>();
  }
}
