// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;

using Smdn.Protocols.Imap4.Client.Session;

namespace Smdn.Protocols.Imap4.WebClient {
  internal class ImapFetchMessageBodyStream : Stream {
    public override bool CanSeek {
      get { return false; }
    }

    public override bool CanRead {
      get { return true; }
    }

    public override bool CanWrite {
      get { return false; }
    }

    public override bool CanTimeout {
      get { return true; }
    }

    public override long Position {
      get { return position; }
      set { throw new NotSupportedException(); }
    }

    public override long Length {
      get { throw new NotSupportedException(); }
    }

    internal ImapFetchMessageBodyStream(ImapSession session, long fetchUid, string fetchSection)
    {
      this.buffer = new byte[8192];
      this.session = session;
      this.fetchUid = fetchUid;
      this.fetchSection = fetchSection;
    }

    public override void Close()
    {
      Dispose(true);
    }

    protected override void Dispose(bool disposing)
    {
      if (disposed)
        return;

      base.Dispose(disposing);

      buffer = null;

      disposed = true;
    }

    public override void SetLength(long value)
    {
      throw new NotSupportedException();
    }

    public override long Seek(long offset, SeekOrigin origin)
    {
      throw new NotSupportedException();
    }

    public override void Flush()
    {
      throw new NotSupportedException();
    }

    public override void Write(byte[] buffer, int offset, int count)
    {
      throw new NotSupportedException();
    }

    public override int Read(byte[] dest, int offset, int count)
    {
      RejectDisposed();

      if (dest == null)
        throw new ArgumentNullException("dest");
      if (offset < 0)
        throw new ArgumentOutOfRangeException("offset", "must be greater than or equals to 0");
      if (count < 0)
        throw new ArgumentOutOfRangeException("count", "must be greater than or equals to 0");
      if (dest.Length < offset + count)
        throw new ArgumentException("invalid range");

      var read = 0;

      for (;;) {
        if (bufRemain == 0) {
          if (eof)
            return read;
          else
            eof = FillBuffer();
        }

        if (count <= bufRemain) {
          Buffer.BlockCopy(buffer, bufOffset, dest, offset, count);

          bufOffset += count;
          bufRemain -= count;
          position  += count;

          read += count;

          return read;
        }
        else {
          Buffer.BlockCopy(buffer, bufOffset, dest, offset, bufRemain);

          offset    += bufRemain;
          count     -= bufRemain;
          position  += bufRemain;

          read += bufRemain;

          bufRemain = 0;
        }
      }
    }

    private bool FillBuffer()
    {
      ImapMessage[] fetchPartial;

      if (session.Fetch(ImapSequenceSet.CreateUidSet(fetchUid), ImapFetchDataItem.BodyPeek(fetchSection, position, buffer.Length), out fetchPartial).Failed)
        throw new IOException();

      var fetchedBody = ImapEncoding.Transfer8Bit.GetBytes(fetchPartial[0].Body);

      Buffer.BlockCopy(fetchedBody, 0, buffer, 0, fetchedBody.Length);

      bufOffset = 0;
      bufRemain = fetchedBody.Length;

      return (fetchedBody.Length < buffer.Length);
    }

    private void RejectDisposed()
    {
      if (disposed)
        throw new ObjectDisposedException(GetType().Name);
    }

    private bool disposed = false;
    private /*readonly*/ ImapSession session;
    private readonly long fetchUid;
    private /*readonly*/ string fetchSection;
    private /*readonly*/ byte[] buffer;
    private long position = 0;
    private int bufOffset = 0;
    private int bufRemain  = 0;
    private bool eof = false;
  }
}
