// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Net;

using Smdn.Protocols.Imap4.Client.Session;

namespace Smdn.Protocols.Imap4.WebClient {
  internal class ImapFetchMessageWebRequest : ImapWebRequest {
    public ImapFetchMessageWebRequest(ImapUri requestUri, ImapSessionManager sessionManager)
      : base(requestUri, sessionManager)
    {
    }

    protected override ImapWebResponse InternalGetResponse(ImapUri requestUri)
    {
      var response = SelectMailbox(requestUri);

      if (response != null)
        return response;

      if (string.IsNullOrEmpty(Method) ||
          string.Equals(Method, ImapWebRequestMethods.Fetch, StringComparison.InvariantCultureIgnoreCase))
        return GetFetchResponse(requestUri);
      else if (string.Equals(Method, ImapWebRequestMethods.Expunge, StringComparison.InvariantCultureIgnoreCase))
        return GetExpungeResponse(requestUri);
      else if (string.Equals(Method, ImapWebRequestMethods.Store, StringComparison.InvariantCultureIgnoreCase))
        return GetStoreResponse(requestUri);
      else
        throw new NotSupportedException(string.Format("unsupported request method: {0}", Method));
    }

    private ImapWebResponse GetFetchResponse(ImapUri requestUri)
    {
      ImapMessage[] messages;

      var result = Session.Fetch(requestUri.GetSequenceSet(), ImapFetchDataItem.BodyStructure + ImapFetchDataItem.Rfc822Size, out messages);

      if (result.Failed)
        return new ImapWebResponse(result);

      if (messages.Length <= 0) // XXX
        return new ImapWebResponse(result);

      var response = new ImapWebResponse(result);

      response.ContentLength = messages[0].Rfc822Size;
      response.FetchMessageStream = new ImapFetchMessageBodyStream(Session, requestUri.Uid, requestUri.Section);

      return response;
    }

    private ImapWebResponse GetExpungeResponse(ImapUri requestUri)
    {
      var uid = requestUri.GetSequenceSet();

      var result = Session.Store(uid, ImapStoreDataItem.AddFlags(ImapMessageFlag.Deleted));

      if (result.Failed)
        return new ImapWebResponse(result);

      if (Session.ServerCapabilities.Has(ImapCapability.UidPlus))
        result = Session.UidExpunge(uid);
      else
        result = Session.Expunge();

      return new ImapWebResponse(result);
    }

    private ImapWebResponse GetStoreResponse(ImapUri requestUri)
    {
      return new ImapWebResponse(Session.Store(requestUri.GetSequenceSet(), StoreDataItem));
    }
  }
}
