// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

namespace Smdn.Protocols.Imap4.WebClient {
  public static class ImapWebRequestMethods {
    // ImapServerWebRequest methods
    public static readonly string Lsub  = "LSUB";
    public static readonly string List  = "LIST";
    public static readonly string NoOp  = "NOOP";

    // ImapMailboxWebRequest methods
    public static readonly string Fetch       = "FETCH";
    public static readonly string Append      = "APPEND";
    public static readonly string Create      = "CREATE";
    public static readonly string Delete      = "DELETE";
    public static readonly string Expunge     = "EXPUNGE";
    public static readonly string Examine     = "EXAMINE";
    public static readonly string Rename      = "RENAME";
    public static readonly string Subscribe   = "SUBSCRIBE";
    public static readonly string Unsubscribe = "UNSUBSCRIBE";

    // ImapFetchMessageWebRequest methods
    // FETCH
    // EXPUNGE
    public static readonly string Store   = "STORE";

    // ImapSearchMessageWebRequest methods
    public static readonly string Search  = "SEARCH";
    public static readonly string Copy    = "COPY";
    // EXPUNGE
    // STORE
    public static readonly string Thread  = "THREAD";
    public static readonly string Sort    = "SORT";

    internal static string GetDefaultMethod(ImapUriForm form)
    {
      switch (form) {
        case ImapUriForm.Server:
          return Lsub;
        case ImapUriForm.ListMessages:
          return Fetch;
        case ImapUriForm.FetchMessage:
          return Fetch;
        case ImapUriForm.SearchMessages:
          return Search;
        default:
          return null;
      }
    }
  }
}
