// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.IO;
using System.Net;

using Smdn.Protocols.Imap4.Client.Session;
using Smdn.Protocols.Imap4.Client.Transaction;
using Smdn.Protocols.Imap4.Client.Protocol;

namespace Smdn.Protocols.Imap4.WebClient {
  public class ImapWebResponse : WebResponse {
    internal ImapSessionManager SessionManager {
      get; set;
    }

    internal ImapSession Session {
      get; set;
    }

    internal bool KeepAlive {
      get; set;
    }

    public ImapNamespace Namespaces {
      get { return Session.Namespaces; }
    }

    public Dictionary<string, string> ServerID {
      get { return new Dictionary<string, string>(Session.ServerID); }
    }

    public ImapCapabilityList ServerCapabilities {
      get { return new ImapCapabilityList(true, Session.ServerCapabilities); }
    }

    private ImapUri responseUri = null;

    public override Uri ResponseUri {
      get { return responseUri; }
    }

    public override long ContentLength {
      get; set;
    }

    public override bool IsFromCache {
      get { return false; }
    }

    public ImapTransactionResult Result {
      get; private set;
    }

    public ImapTransactionResultCode ResultCode {
      get; private set;
    }

    public string ResponseDescription {
      get; private set;
    }

    private WebHeaderCollection headers = new WebHeaderCollection();

    public override WebHeaderCollection Headers {
      get { return headers; }
    }

    public ImapMessage[] Messages {
      get; internal set;
    }

    public ImapMailbox[] Mailboxes {
      get; internal set;
    }

    public ImapThreadTree ThreadTree {
      get; internal set;
    }

    internal ImapFetchMessageBodyStream FetchMessageStream {
      get; set;
    }

    internal ImapWebResponse(ImapTransactionResult r)
      : base()
    {
      this.SessionManager = null;
      this.Session = null;
      this.KeepAlive = true;
      this.ContentLength = 0;
      this.Messages = new ImapMessage[] {};
      this.Mailboxes = new ImapMailbox[] {};
      this.ThreadTree = new ImapThreadTree(true, null, new ImapThreadTree[] {});
      this.FetchMessageStream = null;

      this.Result = r;
      this.ResultCode = r.Code;
      this.ResponseDescription = r.Description;
    }

    public override Stream GetResponseStream()
    {
      if (FetchMessageStream == null)
        //throw new ImapWebException("not found or not message request", WebExceptionStatus.ReceiveFailure);
        return Stream.Null;
      else
        return FetchMessageStream;
    }

    public override void Close()
    {
      if (Session != null) {
        if (!KeepAlive) {
          Session.Disconnect(true);
          SessionManager.UnregisterSession(responseUri);
        }

        Session = null;
      }

      SessionManager = null;

      if (FetchMessageStream != null) {
        FetchMessageStream.Close();
        FetchMessageStream = null;
      }
    }

    internal void SetResponseUri(ImapUri uri)
    {
      responseUri = uri;
    }
  }
}
