// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;

namespace Smdn.Protocols.Imap4 {
  public class ImapNamespaceDesc {
    public string Prefix {
      get { return prefix; }
    }

    public string HierarchyDelimiter {
      get { return hierarchyDelimiter; }
    }

    public Dictionary<string, string[]> Extensions {
      get { return extensions; }
    }

    public ImapNamespaceDesc(string prefix)
      : this(prefix, null, new Dictionary<string, string[]>())
    {
    }

    public ImapNamespaceDesc(string prefix, string hierarchyDelimiter)
      : this(prefix, hierarchyDelimiter, new Dictionary<string, string[]>())
    {
    }

    public ImapNamespaceDesc(string prefix, string hierarchyDelimiter, Dictionary<string, string[]> extensions)
    {
      if (prefix == null)
        throw new ArgumentNullException("prefix");
      if (extensions == null)
        throw new ArgumentNullException("extensions");

      this.prefix = prefix;
      this.hierarchyDelimiter = hierarchyDelimiter;
      this.extensions = extensions;
    }

    public override string ToString()
    {
      return string.Format("{{Prefix={0}, HierarchyDelimiter={1}, Extensions={2}}}",
                           prefix,
                           hierarchyDelimiter,
                           extensions);
    }

    private /*readonly*/ string prefix;
    private /*readonly*/ string hierarchyDelimiter;
    private /*readonly*/ Dictionary<string, string[]> extensions;
  }
}
