// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using Smdn.Protocols.Imap4.Client.Protocol;

namespace Smdn.Protocols.Imap4 {
  public class ImapUriBuilder {
    public string Host {
      get; set;
    }

    public int Port {
      // UriBuilder.Port:
      // ポートが URI の一部として指定されない場合、Port プロパティは値 -1 を返します。
      get; // default ImapUri.DefaultPort
      set; // if < -1 throw ArgumentOutOfRangeException
    }

    public string Query {
      get; // starts with '?'
      set;
    }

    public string UserName {
      get;
      set;
    }

    public ImapAuthenticationMechanism Authentication {
      get;
      set;
    }

    public int? UidValidity {
      get;
      set; // must be non-zero positive number
    }

    public int? MessageUid {
      get;
      set; // must be non-zero positive number
    }

    public string Section {
      get;
      set;
    }

    public string Mailbox {
      get;
      set;
    }

    public ImapUriBuilder()
      : this(string.Empty, string.Empty, null)
    {
    }

    public ImapUriBuilder(ImapUri uri)
    {
    }

    // imap://<iserver>/
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication)
      : this(host, userName, authentication, ImapConnection.ImapDefaultPort)
    {
    }

    // imap://<iserver>:port/
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, int port)
      : this(host, userName, authentication, port, null, null, null)
    {
    }

    // imap://<iserver>/<enc_mailbox>?<enc_search>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, string mailbox, string query)
      : this(host, userName, authentication, ImapConnection.ImapDefaultPort, mailbox, null, query)
    {
    }

    // imap://<iserver>:port/<enc_mailbox>?<enc_search>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, int port, string mailbox, string query)
      : this(host, userName, authentication, port, mailbox, null, query)
    {
    }

    // imap://<iserver>/<enc_mailbox>uidvalidity
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, string mailbox, int uidValidity)
      : this(host, userName, authentication, ImapConnection.ImapDefaultPort, mailbox, uidValidity, string.Empty)
    {
    }

    // imap://<iserver>:port/<enc_mailbox>uidvalidity
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, int port, string mailbox, int uidValidity)
      : this(host, userName, authentication, port, mailbox, uidValidity, string.Empty)
    {
    }
    // imap://<iserver>/<enc_mailbox>uidvalidity?<enc_search>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, string mailbox, int? uidValidity, string query)
      : this(host, userName, authentication, ImapConnection.ImapDefaultPort, mailbox, uidValidity, query)
    {
    }

    // imap://<iserver>:port/<enc_mailbox>uidvalidity?<enc_search>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, int port, string mailbox, int? uidValidity, string query)
    {
      this.Host = host;
      this.UserName = userName;
      this.Authentication = authentication;
      this.Port = port;
      this.Mailbox = mailbox;
      this.UidValidity = uidValidity;
      this.Query = query;

      throw new NotImplementedException();
    }

    // imap://<iserver>/<enc_mailbox>uidvalidity<iuid>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, string mailbox, int? uidValidity, int messageUid)
      : this(host, userName, authentication, ImapConnection.ImapDefaultPort, mailbox, uidValidity, messageUid, string.Empty)
    {
    }

    // imap://<iserver>:port/<enc_mailbox>uidvalidity<iuid>
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, int port, string mailbox, int? uidValidity, int messageUid)
      : this(host, userName, authentication, port, mailbox, uidValidity, messageUid, string.Empty)
    {
    }

    // imap://<iserver>/<enc_mailbox>uidvalidity<iuid>isection
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, string mailbox, int? uidValidity, int? messageUid, string section)
      : this(host, userName, authentication, ImapConnection.ImapDefaultPort, mailbox, uidValidity, messageUid, section)
    {
    }

    // imap://<iserver>:port/<enc_mailbox>uidvalidity<iuid>isection
    public ImapUriBuilder(string host, string userName, ImapAuthenticationMechanism authentication, int port, string mailbox, int? uidValidity, int? messageUid, string section)
    {
      this.Host = host;
      this.UserName = userName;
      this.Authentication = authentication;
      this.Port = port;
      this.Mailbox = mailbox;
      this.UidValidity = uidValidity;
      this.MessageUid = messageUid;
      this.Section = section;

      throw new NotImplementedException();
    }
  }
}
